@@ -78,7 +78,7 @@ class TrainerController:
 
         for brain_name in self.trainers.keys():
             self.trainers[brain_name].save_model()
-        self.logger.info("Saved Model")
+        self.logger.debug("Saved Model")
 
     @staticmethod
     def _create_output_path(output_path):
@@ -137,7 +137,6 @@ class TrainerController:
         policy = trainer.create_policy(
             parsed_behavior_id,
             env_manager.training_behaviors[name_behavior_id],
-            create_graph=True,
         )
         trainer.add_policy(parsed_behavior_id, policy)
 
@@ -171,6 +170,7 @@ class TrainerController:
         try:
             # Initial reset
             self._reset_env(env_manager)
+            self.param_manager.log_current_lesson()
             while self._not_done_training():
                 n_steps = self.advance(env_manager)
                 for _ in range(n_steps):
